<?php
class dsv {

var $daten;

function leseDateiEin($datei, $ergebnisse = '*'){
	# Bestimmen welche Art die Datei enthält und die entsprechende Methode aufrufen
	if(file_exists($datei)){
		$_inhalt = file_get_contents($datei);
	}
	else{
		return false;
	}
	# Kommentare entfernen
	$_inhalt = preg_replace('~\(\*.*?\*\)~s','',$_inhalt);
	$_inhalt = ltrim($_inhalt);
	$bom = pack("CCC", 0xef, 0xbb, 0xbf);
	if (0 == strncmp($_inhalt, $bom, 3)) { # BOM entfernen
	    $_inhalt = substr($_inhalt, 3);
	}
	# Dateiart bestimmen
	$pos = strpos($_inhalt,':');
	$art = trim(strtolower(substr(trim($_inhalt),0,$pos)));
	$dsvversion = 5;
	if($art == 'format'){ # Dateien im DSV6-Format
		$pos = strpos($_inhalt,';',7);
		$art = trim(strtolower(substr($_inhalt,7,$pos-7)));
		$dsvversion = 6;
	}
	else {
		$_inhalt = utf8_encode($_inhalt);
	}

	if($ergebnisse == 'art'){
		return $art;
	}
	switch($art){
	case 'vereinsmeldeliste':
		$ergebnis = $this->_parsevereinsmeldeliste($_inhalt, $ergebnisse, $dsvversion);
		break;
	case 'vereinsergebnisliste':
		$ergebnis = $this->_parseVereinsergebnisliste($_inhalt, $ergebnisse, $dsvversion);
		break;
	case 'wettkampfergebnisliste':
		$ergebnis = $this->_parsewettkampfergebnisliste($_inhalt, $ergebnisse, $dsvversion);
		break;
	case 'wettkampfdefinitionsliste':
		$ergebnis = $this->_parsewettkampfdefinitionsliste($_inhalt, $ergebnisse, $dsvversion);
		break;
	default:
		return false;
		break;
	}
	if(in_array($art, array('vereinsergebnisliste', 'wettkampfergebnisliste')) and isset($ergebnis['wettkaempfe']) and count($ergebnis['wettkaempfe'])>0){
		$wknr = array_keys($ergebnis['wettkaempfe']);
		$this->daten = $ergebnis;
		$this->daten['ergebnisse'] = array();
		foreach($wknr as $nr){
			if(isset($ergebnis['ergebnisse'][$nr])){
				$this->daten['ergebnisse'][$nr] = $ergebnis['ergebnisse'][$nr];
			}
		}
	}
	elseif(in_array($art, array('vereinsmeldeliste', 'wettkampfdefinitionsliste'))){
		$this->daten = $ergebnis;
	}
	return true;
}

function _parsevereinsmeldeliste($daten, $ergebnisse = '*', $dsvversion = 5){
	$erg = array();
	$erg['art'] = 'vereinsmeldeliste';

	if($dsvversion == 6){
		$daten = $this->_parseDsv6Datei($daten);
		
		# Format
		$format = $this->_holeDsv6Daten($daten, 'format');
		$erg['format'] = $format[2];
		$daten = $this->_entferneDsv6Daten($daten, 'format');
		$daten = $this->_entferneDsv6Daten($daten, 'erzeuger');
		
		# Veranstaltungsdaten
		$veranstaltung = $this->_holeDsv6Daten($daten, 'VERANSTALTUNG');
		$erg['veranstaltung'] = array(
			'bezeichnung' => $veranstaltung[1],
			'ort'         => $veranstaltung[2],
			'bahnlaenge'  => $veranstaltung[3],
			'zeitmessung' => $veranstaltung[4],
		);
		$daten = $this->_entferneDsv6Daten($daten, 'VERANSTALTUNG');
		
		# Abschnitte
		$abschnitte = $this->_holeDsv6Daten($daten, 'abschnitt', false);
		foreach($abschnitte as $key => $abschn){
			$erg['abschnitte'][(int)$abschn[1]] = array(
				'datum' => $abschn[2],
				'zeit'  => $this->_sinnloserWert($abschn[3], 'abschnittsbeginn')?false:$abschn[3],
				'einlass' => false
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'abschnitt');
			
		# Wettkämpfe
		$wks = $this->_holeDsv6Daten($daten, 'wettkampf');
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		$erg['wettkaempfe'] = array();
		# die Arrays $staffeln und $einzel dienen dazu um später zu wissen welcher Wettkampf ein Staffel- und welcher ein Einzelwettkampf ist
		$_staffel = $_einzel = array();
		foreach($wks as $key => $wk){
			$erg['wettkaempfe'][$wk[1]] = array(
				'nr'     => $wk[1],
				'wkart'  => $wk[2],
				'abschn' => (int)$wk[3],
				'laenge' => ((!empty($wk[4]) and $wk[4]>1)?$wk[4].'x':'').$wk[5],
				'lage'   => $disziplinumsetzung[$wk[6]],
				'mw'     => strtolower($wk[8]),
				'jahrgaenge' => '*'
			);
			if(!empty($wk[4]) and $wk[4]>1){
				$_staffel[] = $wk[1];
			}
			else{
				$_einzel[] = $wk[1];
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wettkampf');
		
		$verein = $this->_holeDsv6Daten($daten, 'verein', true);
		$erg['verein'] = array(
			'name'           => trim($this->_entferneNl($verein[1])),
			'vnkennzahl'     => $this->_sinnloserWert($verein[2], 'vereinsid')?'':$verein[2],
			'landeschwverb'  => $verein[3],
			'finanatkuerzel' => $this->_sinnloserWert($verein[4], 'finanatkuerzel')?'':$verein[4],
		);
		$daten = $this->_entferneDsv6Daten($daten, 'verein');
		
		$ansprechpartner = $this->_holeDsv6Daten($daten, 'ansprechpartner', true);
		$erg['ansprechpartner'] = array(
			'name'    => $ansprechpartner[1],
			'strasse' => $ansprechpartner[2],
			'plz'     => $ansprechpartner[3],
			'ort'     => $ansprechpartner[4],
			'land'    => $ansprechpartner[5],
			'telefon' => $ansprechpartner[6],
			'fax'     => $this->_sinnloserWert($ansprechpartner[7], 'fax')?'':$ansprechpartner[7],
			'email'   => $ansprechpartner[8]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'ansprechpartner');
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
		
		$erg['kampfrichter'] = array();
		$kampfrichter = $this->_holeDsv6Daten($daten, 'karimeldung', true);
		foreach($kampfrichter as $kari){
			$erg['kampfrichter'][$kari[1]] = array(
				'mid'   => '',
				'name'  => $kari[2],
				'karigr' => $kari[3],
				'_abschnitte'  => array()
			);
		}
		$kariabschnitte = $this->_holeDsv6Daten($daten, 'kariabschnitt', true);
		foreach($kariabschnitte as $abschn){
			$erg['kampfrichter'][$abschn[1]]['_abschnitte'][] = $abschn[2];
		}
		$daten = $this->_entferneDsv6Daten($daten, 'karimeldung');
		$daten = $this->_entferneDsv6Daten($daten, 'kariabschnitt');
		$daten = $this->_entferneDsv6Daten($daten, 'trainer');
		
		$erg['meldungen_e'] = array();
		$personen = $this->_holeDsv6Daten($daten, 'pnmeldung', true);
		foreach($personen as $person){
			$erg['meldungen_e'][$person[3]] = array(
				'mid'   => '',
				'name'  => $person[1],
				'dsvid' => $this->_sinnloserWert($person[2], 'dsvid')?'':$person[2],
				'veranstid' => $person[3],
				'mw'    => strtolower($person[4]),
				'jahrg' => $person[5],
				'alterskl' => $person[6],
				'_starts'  => array()
			);
		}
		$einzelstarts = $this->_holeDsv6Daten($daten, 'startpn', true);
		foreach($einzelstarts as $start){
			$erg['meldungen_e'][$start[1]]['_starts'][$start[2]] = substr($start[3],3);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'pnmeldung');
		$daten = $this->_entferneDsv6Daten($daten, 'startpn');

		$erg['meldungen_s'] = array();
		$staffeln = $this->_holeDsv6Daten($daten, 'stmeldung', true);
		foreach($staffeln as $staffel){
			$erg['meldungen_s'][$staffel[2]] = array(
				'mannschaft'   => $staffel[1],
				'staffelklasse'  => '',
				'_starts'  => array()
			);
		}
		$staffelstarts = $this->_holeDsv6Daten($daten, 'startst', true);
		foreach($staffelstarts as $start){
			$erg['meldungen_s'][$start[1]]['_starts'][$start[2]] = substr($start[3],3);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'stmeldung');
		$daten = $this->_entferneDsv6Daten($daten, 'startst');
		$daten = $this->_entferneDsv6Daten($daten, 'staffelperson');
	}
	else{
		# Ueberschrift, Dateiende
		$daten = str_replace('VEREINSMELDELISTE:','',$daten);
		$daten = str_replace('DATEIENDE','',$daten);
	
		# Format
		preg_match('~\s*FORMAT\s*:\s*(4|5)\s*;\s*(?:(ANSI|IBM)\s*;)?\s*~',$daten,$match);
		$erg['format'] = $match[1];
		$daten = str_replace($match[0],'',$daten);
		# Veranstaltung
		$regex = '~\s*VERANSTALTUNG\s*:\s*([^;]+)\s*;\s*([^;]+)\s*;(?:\s*(25|50|X)\s*;?\s*(HANDZEIT|AUTOMATISCH)\s*;)?\s*~';
		preg_match($regex, $daten, $match);
		$erg['veranstaltung'] = array(
			'bezeichnung' => $match[1],
			'ort' => $match[2],
			'bahnlaenge' => isset($match[3])?$match[3]:'',
			'zeitmessung' => isset($match[4])?$match[4]:'',
		);
		$daten = str_replace($match[0], '', $daten);
		
		# Abschnitte
		$regex = '~ABSCHNITT\s*:\s*(\d+)\s*;\s*(\d{2}\.\d{2}\.\d{4})\s*;\s*((?:\d{2})?:(?:\d{2})?)?\s*;\s*';
		$regex .= '(?:EINLASS\s*:\s*(\d{2}:\d{2})\s*;\s*)?~'; # Einlass (in der aktuellen Version des dsv-Standards nicht mehr enthalten)
		preg_match_all($regex, $daten, $match);
		$erg['abschnitte'] = array();
		foreach($match[1] as $key => $abschn){
			$erg['abschnitte'][(int)$abschn] = array(
				'datum' => $match[2][$key],
				'zeit'  => $this->_sinnloserWert($match[3][$key], 'abschnittsbeginn')?false:$match[3][$key],
				'einlass' => !empty($match[4][$key])?$match[4][$key]:false
			);
		}
		$daten = str_replace($match[0],'',$daten);
	
		# Wettkaempfe
		$regex = '~WETTKAMPF\s*:\s*(\d+)\s*;\s*(?:(V|Z|F|A|E|N)?\s*;\s*)?(\d+)\s*;\s*((?:\d{1,2}\s*[*x]\s*)?\d{1,5})\s*(F|R|B|S|L|X)\s*(W|M|X)\s*;\s*(?:(SW|MS|FS)\s*;\s*)?~';
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		preg_match_all($regex, $daten, $match);
		$erg['wettkaempfe'] = array();
		# die Arrays $staffeln und $einzel dienen dazu um später zu wissen welcher Wettkampf ein Staffel- und welcher ein Einzelwettkampf ist
		$_staffel = $_einzel = array();
		foreach($match[1] as $key => $wk){
			$erg['wettkaempfe'][$wk] = array(
				'nr'     => $match[1][$key],
				'wkart'  => $match[2][$key],
				'abschn' => (int)$match[3][$key],
				'laenge' => str_replace(' ', '', $match[4][$key]),
				'lage'   => $disziplinumsetzung[$match[5][$key]],
				'mw'     => strtolower($match[6][$key])
			);
			if($match[4][$key]{1} == 'x' or ((strlen($match[4][$key])>2) and $match[4][$key]{2} == 'x')){
				$_staffel[] = $wk;
			}
			else{
				$_einzel[] = $wk;
			}
		}
		$daten = str_replace($match[0],'',$daten);
		# Verein
		$regex = '~VEREIN(?:SBEZEICHNUNG)?\s*:\s*([^;]+)\s*;\s*VNKENNZAHL\s*:\s*(\d+)\s*;\s*LSV\s*:\s*(\d{1,2})\s*;\s*NATION\s*:\s*([^;]+)\s*;\s*~';
		preg_match($regex,$daten,$match);
		$erg['verein'] = array(
			'name'           => trim($this->_entferneNl($match[1])),
			'vnkennzahl'     => $this->_sinnloserWert($match[2], 'vereinsid')?'':$match[2],
			'landeschwverb'  => $match[3],
			'finanatkuerzel' => $this->_sinnloserWert($match[4], 'finanatkuerzel')?'':$match[4],
		);
		$daten = str_replace($match[0],'',$daten);
		# Ansprechpartner
		$regex = '~\s*ANSPRECHPARTNER\s*:\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*~';
		preg_match($regex, $daten, $match);
		$erg['ansprechpartner'] = array(
			'name'    => $match[1],
			'strasse' => $match[2],
			'plz'     => $match[3],
			'ort'     => $match[4],
			'land'    => $match[5],
			'telefon' => $match[6],
			'fax'     => $this->_sinnloserWert($match[7], 'fax')?'':$match[7],
			'email'   => $match[8]
		);
		$daten = str_replace($match[0], '', $daten);
	
	
		if($ergebnisse == 'meta'){
			return $erg;
		}
	
		# Kampfrichtermeldungen
		$regex = '~KARIMELDUNG\s*:\s*([^;]+)\s*;\s*([1-4])\s*;\s*ABSCHNITTE\s*:\s*((?:[\d]+\s*;\s*)*)~';
		$regex_abschnitte = '~(?:(\d+)\s*;\s*)~';
		preg_match_all($regex, $daten, $match);
		$erg['kampfrichter'] = array();
		foreach($match[0] as $key => $kari){
			$kampfrichter = array(
				'mid'   => '',
				'name'  => trim($match[1][$key]),
				'karigr' => $match[2][$key],
				'_abschnitte'  => array()
			);
			preg_match_all($regex_abschnitte, $match[3][$key], $match_k);
			foreach($match_k[1] as $abschn){
				$kampfrichter['_abschnitte'][] = $abschn;
			}
			$erg['kampfrichter'][] = $kampfrichter;
			$daten = str_replace($kari,'',$daten);
		}
	
		# Meldungen von Personen
		$regex_pers = '\s*([^;]+)\s*;\s*([0-9]*)\s*;\s*(M|W)\s*;\s*(\d{1,4})\s*;\s*(\d*)\s*;\s*';
		$regex_zeit = '(?:\s*(?:\d{1,2}:\d{2},\d{2})|(?:\d{1,2}h\d{2}:\d{2}(?:,\d{2})*))';
		$regex_starts = '(?:\s*\d+\s*;\s*'.$regex_zeit.'\s*;\s*)';
		$regex = '~PNMELDUNG\s*:'.$regex_pers.'\s*STARTS\s*:\s*('.$regex_starts.'*)\s*~';
		preg_match_all($regex, $daten, $match);
		$erg['meldungen_e'] = array();
		$regex_starts2 = '\s*(\d+)\s*;\s*('.$regex_zeit.')\s*;\s*';
		foreach($match[0] as $key => $pers){
			$mitgl = array(
				'mid'   => '',
				'name'  => trim($match[1][$key]),
				'dsvid' => $this->_sinnloserWert($match[2][$key], 'dsvid')?'':$match[2][$key],
				'mw'    => strtolower($match[3][$key]),
				'jahrg' => $match[4][$key],
				'alterskl' => $match[5][$key],
				'_starts'  => array()
			);
			preg_match_all('~'.$regex_starts2.'~', $match[6][$key], $match_e);
			foreach($match_e[0] as $key2 => $meldung){
				$mitgl['_starts'][$match_e[1][$key2]] = $match_e[2][$key2];
			}
			$erg['meldungen_e'][] = $mitgl;
			$daten = str_replace($pers,'',$daten);
		}
	
		# Meldungen von Staffeln
		$regex_zeit = '(?:\s*(?:\d{1,2}:\d{2},\d{2})|(?:\d{1,2}h\d{2}:\d{2}(?:,\d{2})*))';
		$regex_starts = '(?:\s*\d+\s*;\s*'.$regex_zeit.'\s*;\s*)';
		$regex = '~STMELDUNG\s*:\s*(\d+)\s*;\s*([^;]*)\s*;\s*STARTS\s*:\s*('.$regex_starts.'*)\s*~';
		$regex_starts2 = '\s*(\d+)\s*;\s*('.$regex_zeit.')\s*;\s*';
		$erg['meldungen_s'] = array();
		preg_match_all($regex, $daten, $match);
		foreach($match[3] as $key => $wk){
			$staffel = array(
				'mannschaft' => $match[1][$key],
				'staffelklasse' => $match[2][$key],
				'_starts' => array()
			);
			preg_match_all('~'.$regex_starts2.'~', $match[3][$key], $match2);
			foreach($match2[0] as $key2 => $st){
				$staffel['_starts'][$match2[1][$key2]] = $match2[2][$key2];
			}
			$erg['meldungen_s'][] = $staffel;
			$daten = str_replace($match[0][$key],'',$daten);
		}
	}
	return $erg;
}

function _parseDsv6Datei($daten){
	$zeilen = preg_split('~(\015\012)|(\015)|(\012)~',$daten);
	$ausgabe = array();
	foreach($zeilen as $row){
		$row = trim($row);
		if(empty($row) or $row=='DATEIENDE'){
			continue;
		}
		$zeile = str_getcsv($row, ';', '');
		$teile = explode(':', $zeile[0],2);
		$zeile[0] = $teile[1];
		array_unshift($zeile, $teile[0]);
		$zeile = array_map('trim', $zeile);
		if(end($zeile) == ""){
			array_pop($zeile);
		}
		$ausgabe[] = $zeile;
	}
	return $ausgabe;
}

function _entferneDsv6Daten($daten, $felder){
	if(!is_array($felder)){
		$felder = array($felder);
	}
	foreach($felder as $feld){
		$feld = strtoupper($feld);
		foreach($daten as $key => $row){
			if(trim($row[0]) != $feld){
				continue;
			}
			unset($daten[$key]);
		}
	}
	return $daten;
}

function _holeDsv6Daten($daten, $feld, $immerEineZeileLiefern = true){
	$feld = strtoupper($feld);
	$erg = array();
	foreach($daten as $key => $row){
		if(trim($row[0]) != $feld){
			continue;
		}
		$erg[] = $row;
	}
	if(count($erg) == 0){
		return array();
	}
	if($immerEineZeileLiefern and count($erg) == 1){
		$erg = $erg[0];
	}
	return $erg;
}

function _parseDsv6Wertungen($wertungen, $wkjahr){
	# Zielformat festlegen
	if(count($wertungen) == 1){
		$_ziel = strtolower($wertungen[0][4]);
	}
	else{
		$_ziel = 'jg';
	}
	
	$ausgabe = '*';
	$_minjg = 9999;
	$_maxjg = 0;
	foreach($wertungen as $wertung){
		if($_ziel == 'ak' and ($wertung[5]==$wertung[6] or empty($wertung[5]) or empty($wertung[6]))){
			# Altersklasse verwenden
			$ak = $wertung[5];
			if(empty($wertung[5])){
				$ak = $wertung[6];
			}
			$ak = strtoupper($ak);
			$wert = '*';
			if($ak == 'J'){
				$wert = 'junioren';
			}
			elseif(in_array($ak, array('A', 'B', 'C', 'D', 'E', 'F'))){
				$wert = 'j'.strtolower($ak);
				break;
			}
			elseif(in_array($ak, array('20','25','30','35','40','45','50','55','60','65','70','75','80','85','90','95','100'))){
				$wert = 'ak'.$ak;
				break;
			}
			elseif(preg_match('~^[0-9]{3]\+$~', $ak)){ # Masters-Staffel-Altersklasse
				$wert = $ak;
				break;
			}
			if(empty($wertung[5])){
				$ausgabe = $wert.'+';
			}
			if(empty($wertung[6])){
				$ausgabe = $wert.'-';
			}
			return $ausgabe;		
		}

		if($wertung[4] == 'JG'){
			if($wertung[5] == $wertung[6]){
				if($wertung[5] < $_minjg){
					$_minjg = $wertung[5];
				}
				if($wertung[6] > $_maxjg){
					$_maxjg = $wertung[6];
				}
			}
			elseif(empty($wertung[5]) or $wertung[5]=='0'){
				$_minjg = 0;
				if($wertung[6] > $_maxjg){
					$_maxjg = $wertung[6];
				}
			}
			elseif(empty($wertung[6]) or $wertung[6]=='9999'){
				$_minjg = 9999;
				if($wertung[5] < $_minjg){
					$_minjg = $wertung[5];
				}
			}
			else{
				if($wertung[5] < $_minjg){
					$_minjg = $wertung[5];
				}
				if($wertung[6] > $_maxjg){
					$_maxjg = $wertung[6];
				}
			}
		}
		else{
			# Fall dass Jahrgänge und Altersklassen vorkommen
		}
	}

	if($_maxjg == 0 and $_minjg == 0){
		$_minjg = 9999;
	}
	if($_maxjg == ($wkjahr-20)){
		$_maxjg = 'ak20';
	}

	if($_maxjg == 9999){
		$ausgabe = $_minjg.'-';
	}
	elseif($_minjg == 0){
		$ausgabe = $_maxjg.'+';
	}
	elseif($_maxjg == $_minjg){
		$ausgabe = $_minjg;
	}
	elseif($_maxjg == 0 and $_minjg == 9999){
		$ausgabe = '*';
	}
	else{
		$ausgabe = $_minjg.'-'.$_maxjg;
	}	
	
	return $ausgabe;
}



function _parsewettkampfdefinitionsliste($daten, $ergebnisse = '*', $dsvversion = 5){
	$erg = array();
	$erg['art'] = 'wettkampfdefinitionsliste';
	
	if($dsvversion == 6){
		$daten = $this->_parseDsv6Datei($daten);
		
		# Format
		$format = $this->_holeDsv6Daten($daten, 'format');
		$erg['format'] = $format[2];
		$daten = $this->_entferneDsv6Daten($daten, 'format');
		$daten = $this->_entferneDsv6Daten($daten, 'erzeuger');
		
		# Veranstaltungsdaten
		$veranstaltung = $this->_holeDsv6Daten($daten, 'VERANSTALTUNG');
		$erg['veranstaltung'] = array(
			'bezeichnung' => $veranstaltung[1],
			'ort'         => $veranstaltung[2],
			'bahnlaenge'  => $veranstaltung[3],
			'zeitmessung' => $veranstaltung[4],
		);
		$daten = $this->_entferneDsv6Daten($daten, 'VERANSTALTUNG');
		$ort = $this->_holeDsv6Daten($daten, 'VERANSTALTUNGSORT');
		$erg['veranstaltungsort'] = array(
			'bad'     => $ort[1],
			'strasse' => $ort[2],
			'plz'     => $ort[3],
			'ort'     => $ort[4],
			'land'    => $ort[5],
			'telefon' => $ort[6],
			'fax'     => $this->_sinnloserWert($ort[7], 'fax')?'':$ort[7],
			'email'   => $ort[8]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'VERANSTALTUNGSORT');

		# Ausschreibung
		$internet = $this->_holeDsv6Daten($daten, 'AUSSCHREIBUNGIMNETZ');
		if($internet === false or empty($internet[1])){
			$erg['internetausschreibung'] = false;
		}
		$erg['internetausschreibung'] = $this->_sinnloserWert($internet[1])?false:$internet[1];
		$daten = $this->_entferneDsv6Daten($daten, 'AUSSCHREIBUNGIMNETZ');
		
		$daten = $this->_entferneDsv6Daten($daten, 'veranstalter');
		$daten = $this->_entferneDsv6Daten($daten, 'ausrichter');

		# Meldeadresse
		$meldeadresse = $this->_holeDsv6Daten($daten, 'meldeadresse');
		$erg['meldeadresse'] = array(
			'name'    => $meldeadresse[1],
			'strasse' => $meldeadresse[2],
			'plz'     => $meldeadresse[3],
			'ort'     => $meldeadresse[4],
			'land'    => $meldeadresse[5],
			'telefon' => $meldeadresse[6],
			'fax'     => $this->_sinnloserWert($meldeadresse[7], 'fax')?'':$meldeadresse[7],
			'email'   => $meldeadresse[8]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'meldeadresse');
	
		# Meldeschluss
		$meldeschluss = $this->_holeDsv6Daten($daten, 'meldeschluss');
		$erg['meldeschluss'] = array(
			'datum'   => $meldeschluss[1],
			'uhrzeit' => $meldeschluss[2]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'meldeschluss');

		$daten = $this->_entferneDsv6Daten($daten, 'bankverbindung');
	
		# Besonderes
		$besonderes = $this->_holeDsv6Daten($daten, 'besonderes');
		if(empty($besonderes[1])){
			$erg['besonderes'] = false;
		}
		else{
			$erg['besonderes'] = $this->_sinnloserWert($besonderes[1])?false:$besonderes[1];
		}
		$daten = $this->_entferneDsv6Daten($daten, 'besonderes');
		
		$daten = $this->_entferneDsv6Daten($daten, 'nachweis');

		# Abschnitte
		$abschnitte = $this->_holeDsv6Daten($daten, 'abschnitt', false);
		foreach($abschnitte as $key => $abschn){
			$erg['abschnitte'][(int)$abschn[1]] = array(
				'datum' => $abschn[2],
				'zeit'  => $this->_sinnloserWert($abschn[5], 'abschnittsbeginn')?false:$abschn[5],
				'einlass' => !empty($abschn[3])?$abschn[3]:false
			);
			
		}
		$daten = $this->_entferneDsv6Daten($daten, 'abschnitt');
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
	
		# Wettkämpfe
		$wks = $this->_holeDsv6Daten($daten, 'wettkampf', false);
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		$erg['wettkaempfe'] = array();
		# die Arrays $staffeln und $einzel dienen dazu um später zu wissen welcher Wettkampf ein Staffel- und welcher ein Einzelwettkampf ist
		$_staffel = $_einzel = array();
		foreach($wks as $key => $wk){
			$erg['wettkaempfe'][$wk[1]] = array(
				'nr'     => $wk[1],
				'wkart'  => $wk[2],
				'abschn' => (int)$wk[3],
				'laenge' => ((!empty($wk[4]) and $wk[4]>1)?$wk[4].'x':'').$wk[5],
				'lage'   => $disziplinumsetzung[$wk[6]],
				'mw'     => strtolower($wk[8]),
				'jahrgaenge' => '*'
			);
			if(!empty($wk[4]) and $wk[4]>1){
				$_staffel[] = $wk[1];
			}
			else{
				$_einzel[] = $wk[1];
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wettkampf');

		# Wertungen
		$wertungen = $this->_holeDsv6Daten($daten, 'wertung');
		$_wertungen = array();
		foreach($wertungen as $wertung){
			if(!isset($wertungen[$wertung[1]])){
				$_wertungen[$wertung[1]] = array();
			}
			$_wertungen[$wertung[1]][] = array(
				4 => $wertung[4],
				5 => $wertung[5],
				6 => $wertung[6]
			);
		}
		$wkjahr = substr($erg['abschnitte']['1']['datum'],-4);
		foreach($_wertungen as $wk => $w){
			$erg['wettkaempfe'][$wk]['jahrgaenge'] = $this->_parseDsv6Wertungen($w, $wkjahr);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wertung');
		
		# Pflichtzeiten
		$pflichtzeiten = $this->_holeDsv6Daten($daten, 'pflichtzeit');
		$wkjahr = substr($erg['abschnitte']['1']['datum'],-4);
		foreach($pflichtzeiten as $pflicht){
			if(!isset($erg['pflichtzeiten'][$pflicht[1]])){
				$erg['pflichtzeiten'][$pflicht[1]] = array();
			}
			$wertung = array(
				4 => $pflicht[3],
				5 => $pflicht[4],
				6 => $pflicht[5]
			);
			$jg = $this->_parseDsv6Wertungen(array($wertung,$wertung), $wkjahr);
			$erg['pflichtzeiten'][$pflicht[1]][$jg] = substr($pflicht[6], 3);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'pflichtzeit');

		# Meldegeld
		$meldegeld = $this->_holeDsv6Daten($daten, 'meldegeld', false);
		foreach($meldegeld as $geld){
			$art = strtoupper($geld[1]);
			switch($art){
			case 'MELDEGELDPAUSCHALE':
			case 'MANNSCHAFTMELDEGELD':
				$erg['meldegeld']['meldegeldpauschale'] = $geld[2];
				break;
			
			case 'EINZELMELDEGELD':
				$erg['meldegeld']['einzelmeldegeld'] = $geld[2];
				break;
				
			case 'STAFFELMELDEGELD':
				$erg['meldegeld']['staffelmeldegeld'] = $geld[2];
				break;
				
			case 'WKMELDEGELD':
				if(!isset($erg['meldegeld']['wkmeldegeld'])){
					$erg['meldegeld']['wkmeldegeld'] = array();
				}
				$erg['meldegeld']['wkmeldegeld'][$geld[3]] = $geld[2];
				break;
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'meldegeld');
		
		if(isset($erg['meldegeld']['wkmeldegeld'])){
			if(!isset($erg['meldegeld']['einzelmeldegeld'])){ # häufigsten Betrag bei den Einzelstrecken ermitteln und verwenden
				$betraege = array();
				foreach($_einzel as $einzel){
					$betraege[] = $erg['meldegeld']['wkmeldegeld'][$einzel];
				}
				$betraege = array_count_values($betraege);
				arsort($betraege);
				$einzelmeldegeld = key($betraege);
				$erg['meldegeld']['einzelmeldegeld'] = $einzelmeldegeld;
				foreach($erg['meldegeld']['wkmeldegeld'] as $wk => $betrag){
					if($betrag != $einzelmeldegeld or !in_array($wk, $_einzel)){
						continue;
					}
					unset($erg['meldegeld']['wkmeldegeld'][$wk]);
				}
			}
		
			if(!isset($erg['meldegeld']['staffelmeldegeld'])){ # häufigsten Betrag bei den Staffeln ermitteln und verwenden
				$betraege = array();
				foreach($_staffel as $staffel){
					$betraege[] = $erg['meldegeld']['wkmeldegeld'][$staffel];
				}
				$betraege = array_count_values($betraege);
				arsort($betraege);
				$staffelmeldegeld = key($betraege);
				$erg['meldegeld']['staffelmeldegeld'] = $staffelmeldegeld;
				foreach($erg['meldegeld']['wkmeldegeld'] as $wk => $betrag){
					if($betrag != $staffelmeldegeld or !in_array($wk, $_staffel)){
						continue;
					}
					unset($erg['meldegeld']['wkmeldegeld'][$wk]);
				}
			}
			
			if(count($erg['meldegeld']['wkmeldegeld']) == 0){
				unset($erg['meldegeld']['wkmeldegeld']);
			}
		}
	}
	else {
		# Ueberschrift, Dateiende
		$daten = str_replace('WETTKAMPFDEFINITIONSLISTE:','',$daten);
		$daten = str_replace('DATEIENDE','',$daten);
		# Format
		preg_match('~\s*FORMAT\s*:\s*(4|5)\s*;\s*(?:(ANSI|IBM)\s*;)?\s*~',$daten,$match);
		$erg['format'] = $match[1];
		$daten = str_replace($match[0],'',$daten);
		# Veranstaltung
		$regex = '~\s*VERANSTALTUNG\s*:\s*([^;]+)\s*;\s*([^;]+)\s*;(?:\s*(25|50|X)\s*;?\s*(HANDZEIT|AUTOMATISCH)\s*;)?\s*~';
		preg_match($regex, $daten, $match);
		$erg['veranstaltung'] = array(
			'bezeichnung' => $match[1],
			'ort' => $match[2],
			'bahnlaenge' => isset($match[3])?$match[3]:'',
			'zeitmessung' => isset($match[4])?$match[4]:'',
		);
		$daten = str_replace($match[0], '', $daten);
	
		# Veranstaltungsort
		$regex = '~\s*VERANSTALTUNGSORT\s*:\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*~';
		preg_match($regex, $daten, $match);
		$erg['veranstaltungsort'] = array(
			'bad'     => $match[1],
			'strasse' => $match[2],
			'plz'     => $match[3],
			'ort'     => $match[4],
			'land'    => $match[5],
			'telefon' => $match[6],
			'fax'     => $this->_sinnloserWert($match[7], 'fax')?'':$match[7],
			'email'   => $match[8]
		);
		$daten = str_replace($match[0], '', $daten);
	
	
		# Internetseite
		$regex = '~\s*AUSSCHREIBUNGIMNETZ\s*:\s*([^;]*)\s*;\s*~';
		$anz = preg_match($regex, $daten, $match);
		$erg['internetausschreibung'] = false;
		if($anz > 0){
			$erg['internetausschreibung'] = $this->_sinnloserWert($match[1])?false:$match[1];
			$daten = str_replace($match[0], '', $daten);
		}
	
		# Meldeadresse
		$regex = '~\s*MELDEADRESSE\s*:\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*~';
		preg_match($regex, $daten, $match);
		$erg['meldeadresse'] = array(
			'name'    => $match[1],
			'strasse' => $match[2],
			'plz'     => $match[3],
			'ort'     => $match[4],
			'land'    => $match[5],
			'telefon' => $match[6],
			'fax'     => $this->_sinnloserWert($match[7], 'fax')?'':$match[7],
			'email'   => $match[8]
		);
		$daten = str_replace($match[0], '', $daten);
	
		# Meldeschluss
		$regex = '~MELDESCHLUSS\s*:\s*(\d{2}\.\d{2}\.\d{4})\s*;\s*(\d{2}:\d{2})\s*;\s*~';
		preg_match($regex, $daten, $match);
		$erg['meldeschluss'] = array(
			'datum'   => $match[1],
			'uhrzeit' => $match[2]
		);
		$daten = str_replace($match[0], '', $daten);
	
		# Besonderes
		$regex = '~BESONDERES\s*:\s*([^;]*);\s*~';
		$anz = preg_match($regex, $daten, $match);
		$erg['besonderes'] = false;
		if($anz > 0){
			$erg['besonderes'] = $this->_sinnloserWert($match[1], 'besonderes')?false:$match[1];
			$daten = str_replace($match[0], '', $daten);
		}
	
		# Abschnitte
		$regex = '~ABSCHNITT\s*:\s*(\d+)\s*;\s*(\d{2}\.\d{2}\.\d{4})\s*;\s*((?:\d{2})?:(?:\d{2})?)?\s*;\s*';
		$regex .= '(?:EINLASS\s*:\s*(\d{2}:\d{2})\s*;\s*)?~'; # Einlass (in der aktuellen Version des dsv-Standards nicht mehr enthalten)
		preg_match_all($regex, $daten, $match);
		$erg['abschnitte'] = array();
		foreach($match[1] as $key => $abschn){
			$erg['abschnitte'][(int)$abschn] = array(
				'datum' => $match[2][$key],
				'zeit'  => $this->_sinnloserWert($match[3][$key], 'abschnittsbeginn')?false:$match[3][$key],
				'einlass' => !empty($match[4][$key])?$match[4][$key]:false
			);
		}
		$daten = str_replace($match[0],'',$daten);
		
		if($ergebnisse == 'meta'){
			return $erg;
		}

		# Wettkaempfe
		$regex = '~WETTKAMPF\s*:\s*(\d+)\s*;\s*(?:(V|Z|F|A|E|N)?\s*;\s*)?(\d+)\s*;\s*((?:\d{1,2}\s*[*x]\s*)?\d{1,5})\s*(F|R|B|S|L|X)\s*(W|M|X)\s*;\s*(?:(SW|MS|FS)\s*;\s*)?~';
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		preg_match_all($regex, $daten, $match);
		$erg['wettkaempfe'] = array();
		# die Arrays $staffeln und $einzel dienen dazu um später zu wissen welcher Wettkampf ein Staffel- und welcher ein Einzelwettkampf ist
		$_staffel = $_einzel = array();
		foreach($match[1] as $key => $wk){
			$erg['wettkaempfe'][$wk] = array(
				'nr'     => $match[1][$key],
				'wkart'  => $match[2][$key],
				'abschn' => (int)$match[3][$key],
				'laenge' => str_replace(' ', '', $match[4][$key]),
				'lage'   => $disziplinumsetzung[$match[5][$key]],
				'mw'     => strtolower($match[6][$key]),
				'jahrgaenge' => '*'
			);
			if($match[4][$key]{1} == 'x' or ((strlen($match[4][$key])>2) and $match[4][$key]{2} == 'x')){
				$_staffel[] = $wk;
			}
			else{
				$_einzel[] = $wk;
			}
		}
		$daten = str_replace($match[0],'',$daten);
	
		# Wertungen
		$regex_zeit = '(?:\s*(?:\d{1,2}:\d{2},\d{2}\s*)|(?:\d{1,2}h\d{2}:\d{2}(?:,\d{2})*\s*))';
		$regex = '~(?:WERTUNG|Wertung)\s*:\s*(\d+)\s*;\s*(?:[VZFAEN]?\s*;\s*)?(?:((?:JG\s*:\s*\d+\s*(?:-\s*\d+)?)|(?:\s*AK\s*:\s*(?:\d+|J|A|B|C|D|E)))\s*;\s*)?(?:PFLICHTZEIT\s*:\s*('.$regex_zeit.');)\s*?~';
		preg_match_all($regex, $daten, $match);
		$daten = str_replace($match[0],'',$daten);
		$wertungen = array();
		foreach($match[1] as $key => $wk){
			if(!isset($wertungen[$wk])){
				$wertungen[$wk] = array();
			}
			$wertungen[$wk][] = array(
				'klasse' => $match[2][$key],
				'pflichtzeit' => $match[3][$key]
			);
		}
		$wkjahr = substr($erg['abschnitte']['1']['datum'],-4);
		foreach($wertungen as $wk => $w){
			$wert = $this->_parseWertungen($w, $wkjahr);
			$erg['wettkaempfe'][$wk]['jahrgaenge'] = $wert['jahrgaenge'];
			if($wert['pflichtzeiten'] !== false){
				if(!isset($erg['pflichtzeiten'])){
					$erg['pflichtzeiten'] = array();
				}
				$erg['pflichtzeiten'][$wk] = $wert['pflichtzeiten'];
			}
		}
		# Meldegeld
		$erg['meldegeld'] = array();
		# Meldegeldpauschale
		$regex = '~MELDEGELDPAUSCHALE\s*:\s*(\d{1,},\d{2})\s*;\s*~';
		$anz = preg_match($regex, $daten, $match);
		if($anz > 0){
			$daten = str_replace($match[0],'',$daten);
			$erg['meldegeld']['meldegeldpauschale'] = $match[1];
		}
		# Einzelmeldegeld
		$regex = '~EINZELMELDEGELD\s*:\s*(\d{1,},\d{2})\s*;\s*~';
		$anz = preg_match($regex, $daten, $match);
		if($anz > 0){
			$daten = str_replace($match[0],'',$daten);
			$erg['meldegeld']['einzelmeldegeld'] = $match[1];
		}
		# Staffelmeldegeld
		$regex = '~STAFFELMELDEGELD\s*:\s*(\d{1,},\d{2})\s*;\s*~';
		$anz = preg_match($regex, $daten, $match);
		if($anz > 0){
			$daten = str_replace($match[0],'',$daten);
			$erg['meldegeld']['staffelmeldegeld'] = $match[1];
		}
		# WK-Meldegeld (abweichendes Meldegeld für einzelne Wettkämpfe
		$regex = '~WKMELDEGELD\s*:\s*(\d+)\s*;\s*(\d{1,},\d{2})\s*;\s*~';
		$anz = preg_match_all($regex, $daten, $match);
		if($anz > 0){
			$daten = str_replace($match[0],'',$daten);
			$wkmeldegeld = array();
			foreach($match[1] as $key => $wk){
				$wkmeldegeld[$wk] = $match[2][$key];
			}
			$betraege = array();
			if(!isset($erg['meldegeld']['staffelmeldegeld'])){ # häufigsten Betrag bei den Staffeln ermitteln und verwenden
				$betraege = array();
				foreach($_staffel as $wknr){
					if(!isset($wkmeldegeld[$wknr])){
						continue;
					}
					$betrag = $wkmeldegeld[$wknr];
					if(!isset($betraege[$betrag])){
						$betraege[$betrag] = 1;
					}
					else{
						$betraege[$betrag]++;
					}
				}
				arsort($betraege);
				$staffel = key($betraege);
				$erg['meldegeld']['staffelmeldegeld'] = $staffel;
				foreach($wkmeldegeld as $wk => $betrag){
					if(!in_array($wk, $_staffel)){
						continue;
					}
					if($betrag != $staffel){
						$erg['meldegeld']['wkmeldegeld'][$wk] = $betrag; 
					}
					else{
						unset($wkmeldegeld[$wk]);
					}
				}
			}
			else{
				foreach($wkmeldegeld as $wk => $betrag){
					if(!in_array($wk, $_staffel)){
						continue;
					}
					if($betrag != $erg['meldegeld']['staffelmeldegeld']){
						$erg['meldegeld']['wkmeldegeld'][$wk] = $betrag; 
					}
					
				}
			}
			
			if(!isset($erg['meldegeld']['einzelmeldegeld'])){ # häufigsten Betrag bei den Einzelstarts ermitteln und verwenden
				$betraege = array();
				foreach($_einzel as $wknr){
					if(!isset($wkmeldegeld[$wknr])){
						continue;
					}
					$betrag = $wkmeldegeld[$wknr];
					if(!isset($betraege[$betrag])){
						$betraege[$betrag] = 1;
					}
					else{
						$betraege[$betrag]++;
					}
				}
				arsort($betraege);
				$einzel = key($betraege);
				$erg['meldegeld']['einzelmeldegeld'] = $einzel;
	
	
				foreach($wkmeldegeld as $wk => $betrag){
					if(!in_array($wk, $_einzel)){
						continue;
					}
					if($betrag != $einzel){
						$erg['meldegeld']['wkmeldegeld'][$wk] = $betrag; 
					}
					else{
						unset($wkmeldegeld[$wk]);
					}
				}
			}
			else{
				foreach($wkmeldegeld as $wk => $betrag){
					if(!in_array($wk, $_einzel)){
						continue;
					}
					if($betrag != $erg['meldegeld']['einzelmeldegeld']){
						$erg['meldegeld']['wkmeldegeld'][$wk] = $betrag; 
					}
					
				}
			}
		}
	}
	# Ergebnisarray zurueckgeben
	return $erg;
}


function _parseWertungen($wertungen, $wkjahr){
	global $_altersklassen;
	$_pflichtzeiten = false;
	$_min = 9999;
	$_max = 0;
	#print_r($wertungen);
	foreach($wertungen as $key => $wertung){
		if(!empty($wertung['pflichtzeit'])){
			$_pflichtzeiten = true;
		}
		#print_r($wertung);
		if($wertung['klasse'][0] == 'J'){ # Jahrgang
			$wert = trim(substr($wertung['klasse'],3));
			if(strpos($wert, '-') !== false){ # Bereich
				$teile = explode('-', $wert);
				$teile[0] = trim($teile[0]);
				$teile[1] = trim($teile[1]);
				$min = (int)min($teile);
				$max = (int)max($teile);
			}
			else{
				$min = $max = (int)$wert;
			}
		}
		else{ # AK
			$wert = trim(substr($wertung['klasse'],3));
			if(!in_array($wert, array('J', 'A', 'B', 'C', 'D', 'E', '20', '25', '30', '35', '40', '45', '50', '55', '60', '65', '70', '75', '80', '85', '90', '95', '100+'))){
				# Staffelalter -> ignorieren
				break;
			}
			else{
				if($wert == 'J'){
					$werte = $_altersklassen['junioren'];
				}
				elseif(strlen($wert) == 1){
					$werte = $_altersklassen['j'.strtolower($wert)];
				}
				else{
					$werte = $_altersklassen['ak'.$wert];
				}
				#echo $werte.'<br>';
				$alter = explode(',', $werte);
				#print_r($alter);
				$min = $wkjahr-max($alter);
				$max = $wkjahr-min($alter);
				#echo $min.'-'.$max.'<br>';
			}
		}
		$wertungen[$key]['min'] = ($min <= ($wkjahr-95))?'':$min;
		$wertungen[$key]['max'] = ($max >= $wkjahr)?'':$max;

		if($_min > $min){
			$_min = $min;
		}
		if($_max < $max){
			$_max = $max;
		}
	}	
	if($_min <= ($wkjahr-95)){
		$_min = '';
	}
	if($_max >= $wkjahr){
		$_max = '';
	}

	if(empty($_min)){
		if(empty($_max)){ # offene Wertung
			$_jahrg = '*';
		}
		else{ # X und älter
			if($_max == ($wkjahr-20)){ # Masters
				$_jahrg = 'ak20+';
			}
			else{
				$_jahrg = $_max.'+';
			}
		}
	}
	elseif(empty($_max)){ # X und jünger
		$_jahrg = $_min.'-';
	}
	else{ # X bis Y
		$_jahrg = $_max.'-'.$_min;
	}

	if(!$_pflichtzeiten){
		return array('jahrgaenge' => $_jahrg, 'pflichtzeiten' => false);
	}

	$pflichtzeiten = array();
	$_ak = array();
	foreach($_altersklassen as $key => $alter){
		$teile = explode(',', $alter);
		$min = $wkjahr-max($teile);
		$max = $wkjahr-min($teile);
		$_ak[$key] = array('min' => $min, 'max' => $max);
	}
	foreach($wertungen as $wertung){
		$key = '*';
		if(empty($wertung['min'])){
			if(empty($wertung['max'])){ # offene Wertung
				$key = '*';
			}
			else{ # X und älter
				if($wertung['max'] == ($wkjahr-20)){ # Masters
					$key = 'ak20+';
				}
				else{
					$key = $wertung['max'].'+';
				}
			}
		}
		elseif(empty($wertung['max'])){ # X und jünger
			$key = $wertung['min'].'-';
		}
		elseif($wertung['min'] == $wertung['max']){ # ein Jahrgang
			$key = $wertung['min'];
		}
		else{ # X bis Y
			# auf einzelne Altersgruppen prüfen (mehrere Altersklassen und/oder Jahrgänge werden nicht
			#   unterstützt, wenn das aufteten sollte, wird die offene Wertung verwendet.)
			foreach($_ak as $ak => $jg){
				if($wertung['min'] != $jg['min'] or $wertung['min'] != $jg['min']){
					continue;
				}
				$key = $ak;
				break;
			}
		}
		if(empty($wertung['pflichtzeit'])){
			continue;
		}
		$pflichtzeiten[$key] = $wertung['pflichtzeit'];
	}
	return array('jahrgaenge' => $_jahrg, 'pflichtzeiten' => $pflichtzeiten);
}


function _sinnloserWert($wert, $art = ''){
	# erkennt sinnlose Werte (wie »???« oder »kein fax«)
	if($wert != '0' and empty($wert)){
		return true;
	}

	if($art == 'dsvid'){
		if(strlen($wert) != 6 or $wert == '0' or $wert == '000000' or $wert == '999999' or !preg_match('~^[0-9]{6}$~',$wert)){
			return true;
		}
		else{
			return false;
		}
	}
	
	if($art == 'vereinsid'){
		if(strlen($wert) != 4 or $wert == '0' or $wert == '0000' or $wert == '9999'){
			return true;
		}
		else{
			return false;
		}
	}
	
	if($art == 'finanatkuerzel'){
		if(strlen($wert) != 3 or !preg_match('~^[A-Z]{3}$~',$wert)){
			return true;
		}
		else{
			return false;
		}
	}

	if($art == 'fax' and strtolower($wert) == 'kein fax'){
		return true;
	}

	if($art == 'abschnittsbeginn' and (trim($wert) == ':' or $wert == '00:00')){
		return true;
	}

	$laenge = strlen($wert);
	$gleichezeichen = true;
	for($i=0;$i<$laenge;$i++){
		if($wert[0] != $wert[$i]){
			$gleichezeichen = false;
		}
	}

	if($laenge >=2 and $gleichezeichen){
		return true;
	}

	return false;
}


function _parseVereinsergebnisliste($daten, $ergebnisse = '*', $dsvversion = 5){
	$erg = array();
	$erg['art'] = 'vereinsergebnisliste';
	
	if($dsvversion == 6){
		$daten = $this->_parseDsv6Datei($daten);
		
		# Format
		$format = $this->_holeDsv6Daten($daten, 'format');
		$erg['format'] = $format[2];
		$daten = $this->_entferneDsv6Daten($daten, 'format');
		$daten = $this->_entferneDsv6Daten($daten, 'erzeuger');
		
		# Veranstaltungsdaten
		$veranstaltung = $this->_holeDsv6Daten($daten, 'veranstaltung');
		$erg['veranstaltung'] = array(
			'bezeichnung' => $veranstaltung[1],
			'ort'         => $veranstaltung[2],
			'bahnlaenge'  => $veranstaltung[3],
			'zeitmessung' => $veranstaltung[4],
		);
		$daten = $this->_entferneDsv6Daten($daten, 'veranstaltung');
		
		# Veranstalter, Ausrichter, Absender
		$veranstalter = $this->_holeDsv6Daten($daten, 'veranstalter');
		$erg['veranstalter'] = $veranstalter[1];
		$daten = $this->_entferneDsv6Daten($daten, 'veranstalter');
		$ausrichter = $this->_holeDsv6Daten($daten, 'ausrichter');
		$erg['ausrichter'] = $ausrichter[1];
		$erg['absender'] = array(
			'name'    => $ausrichter[2],
			'strasse' => $ausrichter[3],
			'plz'     => $ausrichter[4],
			'ort'     => $ausrichter[5],
			'land'    => $ausrichter[6],
			'telefon' => $ausrichter[7],
			'fax'     => $ausrichter[8],
			'email'   => $ausrichter[9]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'ausrichter');
		
		# Abschnitte
		$abschnitte = $this->_holeDsv6Daten($daten, 'abschnitt', false);
		$erg['abschnitte'] = array();
		foreach($abschnitte as $abschn){
			$erg['abschnitte'][$abschn[1]] = array(
				'datum' => $abschn[2],
				'zeit'  => $abschn[3]
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'abschnitt');
				
		$daten = $this->_entferneDsv6Daten($daten, 'kampfgericht');

		# Wettkaempfe
		$wettkaempfe = $this->_holeDsv6Daten($daten, 'wettkampf', false);
		$disziplinumsetzung = array('F' => 1,'R' => 3,'B' => 2,'S' => 4,'L' => 5,'X' => 6);
		$erg['wettkaempfe'] = array();
		$_staffel = array();
		$_einzel = array();
		foreach($wettkaempfe as $wk){
			if($wk[5] == 0 and $wk[6] == 0){
				continue;
			}
			$erg['wettkaempfe'][$wk[1]] = array(
				'nr'     => $wk[1],
				'art'    => $wk[2],
				'abschn' => $wk[3],
				'laenge' => ((!empty($wk[4]) and $wk[4]>1)?$wk[4].'x':'').$wk[5],
				'lage'   => $disziplinumsetzung[$wk[6]],
				'mw'     => strtolower($wk[8]),
				'blzuordn' => $wk[9]
			);
			if(!empty($wk[4]) and $wk[4]>1){
				$_staffel[] = $wk[1];
			}
			else{
				$_einzel[] = $wk[1];
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wettkampf');
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wertung');
		
		# Vereinsbezeichnung
		$verein = $this->_holeDsv6Daten($daten, 'verein', true);
		$erg['vereinsbezeichnung'] = array(
			0 => array(
				'name'           => 'Sonstige (nicht zuzuordnen)',
				'vnkennzahl'     => '9999',
				'landeschwverb'  => '99',
				'finanatkuerzel' => 'GER'
			),
			1 => array(
				'name'           => $verein[1],
				'vnkennzahl'     => $verein[2],
				'landeschwverb'  => $verein[3],
				'finanatkuerzel' => $verein[4]
			)
		);
		$daten = $this->_entferneDsv6Daten($daten, 'verein');

		# Personen
		$personen = $this->_holeDsv6Daten($daten, 'person', false);
		$_personen = array();
		foreach($personen as $person){
			$_personen[$person[3]] = array(
				'name' => $person[1],
				'dsvid' => $person[2],
				'veranstid' => $person[3],
				'mw' => strtolower($person[4]),
				'jahrg' => $person[5],
				'alterskl' => $person[6]
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'person');
		
		$erg['ergebnisse'] = array();
		$einzelergebnisse = $this->_holeDsv6Daten($daten, 'personenergebnis', false);
		foreach($einzelergebnisse as $ergebnis){
			if(!isset($erg['ergebnisse'][$ergebnis[2]])){
				$erg['ergebnisse'][$ergebnis[2]] = array();
			}
			if(!isset($erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]])){
				$erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]] = array();
			}
			$row = $_personen[$ergebnis[1]];
			$row['platz'] = ($ergebnis[5]==0?$ergebnis[7]:$ergebnis[5]);
			$row['verein'] = 1;
			$row['vereinid'] = $erg['vereinsbezeichnung'][1]['vnkennzahl'];
			$row['zwischenzeiten'] = array();
			$row['endzeit'] = substr($ergebnis[6],3);
			$row['rekord'] = '';
			$row['dsbem'] = $ergebnis[8];
			$row['anzahl'] = '';
			$erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]][] = $row;
		}
		$daten = $this->_entferneDsv6Daten($daten, 'personenergebnis');
		
		$zwzeiten = $this->_holeDsv6Daten($daten, 'pnzwischenzeit', false);
		foreach($zwzeiten as $zwzeit){
			foreach($erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $zwzeit[1]){
					continue;
				}
				$erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]][$key]['zwischenzeiten'][$zwzeit[4]] = substr($zwzeit[5],3);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'pnzwischenzeit');
		$daten = $this->_entferneDsv6Daten($daten, 'pnreaktion');
	
		$staffeln = $this->_holeDsv6Daten($daten, 'staffel', false);
		$_staffeln = array();
		foreach($staffeln as $staffel){
			$_staffeln[$staffel[2]] = $staffel[1];
		}
		$daten = $this->_entferneDsv6Daten($daten, 'staffel');
	
		$staffelergebnisse = $this->_holeDsv6Daten($daten, 'staffelergebnis', false);
		foreach($staffelergebnisse as $ergebnis){
			if(!isset($erg['ergebnisse'][$ergebnis[2]])){
				$erg['ergebnisse'][$ergebnis[2]] = array();
			}
			if(!isset($erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]])){
				$erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]] = array();
			}
			$erg['ergebnisse'][$ergebnis[2]][$ergebnis[3]][] = array(
				'platz' => $ergebnis[5]==0?$ergebnis[7]:$ergebnis[5],
				'mannschaft'  => $_staffeln[$ergebnis[1]],
				'staffelklasse' => '',
				'veranstid' => $ergebnis[1],
				'verein' => 1,
				'vereinid' => $erg['vereinsbezeichnung'][1]['vnkennzahl'],
				'zwischenzeiten' => array(),
				'endzeit' => substr($ergebnis[6],3),
				'schwimmer' => array(),
				'rekord' => '',
				'dsbem' => $ergebnis[8],
				'anzahl' => ''
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'staffelergebnis');
		
		$staffelpersonen = $this->_holeDsv6Daten($daten, 'staffelperson', false);
		foreach($staffelpersonen as $person){
			foreach($erg['ergebnisse'][$person[2]][$person[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $person[1]){
					continue;
				}
				$erg['ergebnisse'][$person[2]][$person[3]][$key]['schwimmer'][$person[6]] = array(
					'name'  => $person[4],
					'dsvid' => $person[5],
					'mw'    => strtolower($person[7]),
					'jahrg' => $person[8],
					'alterskl' => $person[9]
				);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'staffelperson');
		
		$zwzeiten = $this->_holeDsv6Daten($daten, 'stzwischenzeit', false);
		foreach($zwzeiten as $zwzeit){
			foreach($erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $zwzeit[1]){
					continue;
				}
				$erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]][$key]['zwischenzeiten'][$zwzeit[5]] = substr($zwzeit[6],3);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'stzwischenzeit');
		$daten = $this->_entferneDsv6Daten($daten, 'stabloese');
	}
	else{
		# Ueberschrift, Dateiende
		$daten = str_replace('VEREINSERGEBNISLISTE:','',$daten);
		$daten = str_replace('DATEIENDE','',$daten);
		# Format
		preg_match('~\s*FORMAT\s*:\s*(4|5)\s*;\s*(?:(ANSI|IBM)\s*;)?\s*~',$daten,$match);
		$erg['format'] = $match[1];
		$daten = str_replace($match[0],'',$daten);
		# Veranstaltung
		$regex = '~\s*VERANSTALTUNG\s*:\s*([^;]+)\s*;\s*([^;]+)\s*;(?:\s*(25|50|X)\s*;?\s*(HANDZEIT|AUTOMATISCH)\s*;)?\s*~';
		preg_match($regex, $daten, $match);
		if(isset($match[1])){
			$erg['veranstaltung'] = array(
				'bezeichnung' => $match[1],
				'ort'         => $match[2],
				'bahnlaenge'  => isset($match[3])?$match[3]:'',
				'zeitmessung' => isset($match[4])?$match[4]:''
			);
		}
		else{
			$erg['veranstaltung'] = array(
				'bezeichnung' => '[unbekannt]',
				'ort' => '',
				'bahnlaenge' => '',
				'zeitmessung' => ''
			);
		}
		$daten = str_replace($match[0], '', $daten);
		# Veranstalter, Ausrichter, Absender
		$regex = '~\s*VERANSTALTER\s*:\s*([^;]+)\s*;\s*AUSRICHTER\s*:\s*([^;]+)\s*;'; # Veranstalter, Absender
		$regex .= '\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*~'; # Adresse
		preg_match($regex, $daten, $match);
		$erg['veranstalter'] = $match[1];
		$erg['ausrichter'] = $match[2];
		$erg['absender'] = array(
			'name' => $match[3],
			'strasse' => $match[4],
			'plz' => $match[5],
			'ort' => $match[6],
			'land' => $match[7],
			'telefon' => $match[8],
			'fax' => $match[9],
			'email' => $match[10]
		);
		$daten = str_replace($match[0], '', $daten);
		# Abschnitte
		preg_match_all('~ABSCHNITT\s*:\s*(\d+)\s*;\s*(\d{2}\.\d{2}\.\d{4})\s*;\s*(\d{2}:\d{2})?\s*;~',$daten,$match);
		$erg['abschnitte'] = array();
		foreach($match[1] as $key => $abschn){
			$erg['abschnitte'][$abschn] = array(
				'datum' => $match[2][$key],
				'zeit'  => $match[3][$key]
			);
		}
		$daten = str_replace($match[0],'',$daten);
		$daten = trim($daten);
		# Wettkaempfe
		$regex = '~WETTKAMPF\s*:\s*(\d+)\s*;\s*(V|Z|F|A|E|N)?\s*;\s*(\d+)\s*;\s*((?:\d{1,2}x)?\d{1,5})\s*(F|R|B|S|L|X)\s*(W|M|X)\s*;';
		if($erg['format'] == '5'){
			$regex .= '\s*(SW|MS|FS)\s*;';
		}
		$regex .= '\s*~';
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		preg_match_all($regex,$daten,$match);
		$erg['wettkaempfe'] = array();
		$staffeln = array();
		$einzel = array();
		foreach($match[1] as $key => $wk){
			$erg['wettkaempfe'][$wk] = array(
				'nr'     => $match[1][$key],
				'art'    => $match[2][$key],
				'abschn' => $match[3][$key],
				'laenge' => $match[4][$key],
				'lage'   => $disziplinumsetzung[$match[5][$key]],
				'mw'     => strtolower($match[6][$key])
			);
			if($match[4][$key]{1} == 'x' or ((strlen($match[4][$key])>2) and $match[4][$key]{2} == 'x')){
				$staffeln[] = $wk;
			}
			else{
				$einzel[] = $wk;
			}
			if($erg['format'] == '5'){
				$erg['wettkaempfe'][$wk]['blzuordn'] = $match[7][$key];
			}
		}
		$daten = str_replace($match[0],'',$daten);
		$daten = trim($daten);
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
		
		# Verein
		$regex = '~VEREIN(?:SBEZEICHNUNG)?\s*:\s*([^;]+)\s*;\s*VNKENNZAHL\s*:\s*(\d{1,4})\s*;\s*LSV\s*:\s*(\d{1,2})\s*;\s*NATION\s*:\s*([^;]+)\s*;\s*~';
		preg_match($regex,$daten,$match);
		$match[1] = trim($match[1]);
		$erg['vereinsbezeichnung'][1] = array(
			'name'           => trim($this->_entferneNl($match[1])),
			'vnkennzahl'     => $match[2],
			'landeschwverb'  => $match[3],
			'finanatkuerzel' => $match[4]
		);
		$daten = str_replace($match[0],'',$daten);
	
		# Personenergebnisse
		$regex_pers = '\s*([^;]+)\s*;\s*([0-9]*)\s*;\s*(M|W)\s*;\s*(\d{1,4})\s*;\s*(\d*)\s*;\s*';
		$regex_zeit = '(?:\s*(?:\d{1,2}:\d{2},\d{2}\s*;)|(?:\d{1,2}h\d{2}:\d{2}(?:,\d{2})*\s*;))';
		$regex_endzeit = substr($regex_zeit,0,-1).'\s*(?:REKORD\s*:\s*D[JM]?R\s*;)?\s*)';
		
		$regex_erg = '(?:WK\s*:\s*\d*\s*;\s*[V|Z|F|A|E|N]\s*;\s*(?:\d*|DS|NA|AB|AU|ZU)\s*;\s*'.$regex_zeit.'*\s*'.$regex_endzeit.'\s*(?:DSBEM\s*:\s*[^;]*\s*;\s*)?\s*)';
		$regex = '~PNERGEBNIS\s*:'.$regex_pers.'('.$regex_erg.'*)\s*~';
		preg_match_all($regex, $daten, $match);
		$erg['ergebnisse'] = array();
		$regex_erg2 = '(?:WK\s*:\s*(\d*)\s*;\s*([V|Z|F|A|E|N])\s*;\s*(\d*|DS|NA|AB|AU|ZU)\s*;\s*('.$regex_zeit.'*)\s*('.$regex_zeit.')\s*';
		$regex_erg2 .= '(?:DSBEM\s*:\s*([^;]*)\s*;\s*)?\s*)';
		foreach($match[0] as $key => $pers){
			$mitgl = array(
				'platz' => '',
				'name' => trim($match[1][$key]),
				'dsvid' => $this->_sinnloserWert($match[2][$key], 'dsvid')?'':$match[2][$key],
				'mw' => strtolower($match[3][$key]),
				'jahrg' => $match[4][$key],
				'alterskl' => $match[5][$key],
				'verein' => 1,
				'zwischenzeiten' => array(),
				'endzeit' => '',
				'dsbem' => ''
			);
			preg_match_all('~'.$regex_erg2.'~',$match[6][$key],$match_e);
			foreach($match_e[0] as $key2 => $ergebn){
				if(!isset($erg['ergebnisse'][$match_e[1][$key2]])){
					$erg['ergebnisse'][$match_e[1][$key2]] = array();
				}
				if(!isset($erg['ergebnisse'][$match_e[1][$key2]][$match_e[2][$key2]])){
					$erg['ergebnisse'][$match_e[1][$key2]][$match_e[2][$key2]] = array();
				}
				$tmp = $mitgl;
				$tmp['platz'] = $match_e[3][$key2];
				$tmp['zwischenzeiten'] = explode(';',substr($match_e[4][$key2],0,-1));
				$tmp['endzeit'] = substr($match_e[5][$key2],0,-1);
				$tmp['dsbem'] = preg_replace("/(\015\012)|(\015)|(\012)|( {2,})/",' ',trim($match_e[6][$key2]));
				$erg['ergebnisse'][$match_e[1][$key2]][$match_e[2][$key2]][] = $tmp;
			}
			$daten = str_replace($pers,'',$daten);
		}
		# Staffelergebnisse
		$regex = '~STERGEBNIS\s*:\s*(\d+)\s*;\s*((?:(?:JG:\d+(?:-\d+)?)|(?:AK:(?:\d+\+?|J|A|B|C|D|E)))?)\s*;\s*'; # Staffeldaten
		$regex .= '((?:WK\s*:\s*\d*\s*;\s*[V|Z|F|A|E|N]\s*;\s*(?:\d*|DS|NA|AB|AU|ZU)\s*;';
		$regex .= ''.$regex_zeit.'*\s*'.$regex_endzeit.'\s*'; # Zeiten
		$regex .= '(?:\s*[^;]+\s*;\s*[0-9]*\s*;\s*(?:M|W)\s*;\s*\d{1,4}\s*;\s*\d*\s*;\s*)*'; # Personen
		$regex .= '\s*(?:DSBEM\s*:\s*[^;]+\s*;\s*)?)*)'; # dsbem
		$regex .= '\s*~s';
		preg_match_all($regex, $daten, $match);
		foreach($match[3] as $key => $wk){
			$staffel = array(
				'platz' => '',
				'mannschaft' => $match[1][$key],
				'staffelklasse' => $match[2][$key],
				'verein' => 1,
				'zwischenzeiten' => array(),
				'endzeit' => '',
				'schwimmer' => array(),
				'dsbem' => ''
			);
			$regex2 = '~WK\s*:\s*(\d*)\s*;\s*([V|Z|F|A|E|N])\s*;\s*(\d*|DS|NA|AB|AU|ZU)\s*;';
			$regex2 .= '('.$regex_zeit.'*)\s*('.$regex_zeit.')\s*'; # Zeiten
			$regex2 .= '((?:\s*[^;]+\s*;\s*[0-9]*\s*;\s*(?:M|W)\s*;\s*\d{1,4}\s*;\s*\d*\s*;\s*)*)\s*'; # Personen
			$regex2 .= '\s*((?:DSBEM\s*:\s*[^;]+\s*;\s*)?)\s*~'; # dsbem
			preg_match_all($regex2, $wk, $match2);
			foreach($match2[6] as $key2 => $pers){
				$tmp = $staffel;
				$tmp['platz'] = $match2[3][$key2];
				$tmp['zwischenzeiten'] = explode(';',substr($match2[4][$key2],0,-1));
				$tmp['zwischenzeiten'] = array_map('trim',$tmp['zwischenzeiten']);
				$tmp['endzeit'] = substr($match2[5][$key2],0,-1);
				$tmp['dsbem'] = preg_replace("/(\015\012)|(\015)|(\012)|( {2,})/",' ',trim($match2[7][$key2]));
				$regex3 = '~\s*([^;]+)\s*;\s*([0-9]*)\s*;\s*(M|W)\s*;\s*(\d{1,4})\s*;\s*(\d*)\s*;\s*~'; # Personen
				preg_match_all($regex3, $pers, $match3);
				foreach($match3[1] as $key3 => $name){
					$tmp['schwimmer'][] = array(
						'name'     => trim($name),
						'dsvid'    => $this->_sinnloserWert($match3[2][$key3], 'dsvid')?'':$match3[2][$key3],
						'mw'       => strtolower($match3[3][$key3]),
						'jahrg'    => $match3[4][$key3],
						'alterskl' => $match3[5][$key3],
					);
				}
				if(!isset($erg['ergebnisse'][$match2[1][$key2]])){
					$erg['ergebnisse'][$match2[1][$key2]] = array();
				}
				if(!isset($erg['ergebnisse'][$match2[1][$key2]][$match2[2][$key2]])){
					$erg['ergebnisse'][$match2[1][$key2]][$match2[2][$key2]] = array();
				}
				$erg['ergebnisse'][$match2[1][$key2]][$match2[2][$key2]][] = $tmp;
			}
		}
	}
	#printr($erg);exit;
	return $erg;
}


function _entferneNl($wert){
	return trim(preg_replace('~\s+~',' ',$wert));
}


function _sucheVerein($verein, $vereine){
	# sucht $verein als Key in $vereine und liefert den Wert des entsprechenden Eintrags
	$verein = $this->_entferneNl($verein);
	# Verein heißt genauso wie im Kopf der Datei
	if(isset($vereine[$verein])){
		return $vereine[$verein];
	}
	# Verein ist bei den Ergebnissen gekürzt worden
	foreach($vereine as $v => $id){
		if($verein == substr($v, 0, strlen($verein))){
			return $id;
		}
	}
	# Verein ist im Kopf der DSV-Datei gekürzt worden
	foreach($vereine as $v => $id){
		if($v == substr($verein, 0, strlen($v))){
			return $id;
		}
	}
	# Verein nicht zuzuordnen
	return 0;
}

function _parsewettkampfergebnisliste($daten, $ergebnisse = '*', $dsvversion = 5){
	$erg = array();
	$erg['art'] = 'wettkampfergebnisliste';
	
	if($dsvversion == 6){
		$daten = $this->_parseDsv6Datei($daten);
		
		# Format
		$format = $this->_holeDsv6Daten($daten, 'format');
		$erg['format'] = $format[2];
		$daten = $this->_entferneDsv6Daten($daten, 'format');
		$daten = $this->_entferneDsv6Daten($daten, 'erzeuger');
		
		# Veranstaltungsdaten
		$veranstaltung = $this->_holeDsv6Daten($daten, 'veranstaltung');
		$erg['veranstaltung'] = array(
			'bezeichnung' => $veranstaltung[1],
			'ort'         => $veranstaltung[2],
			'bahnlaenge'  => $veranstaltung[3],
			'zeitmessung' => $veranstaltung[4],
		);
		$daten = $this->_entferneDsv6Daten($daten, 'veranstaltung');
		
		# Veranstalter, Ausrichter, Absender
		$veranstalter = $this->_holeDsv6Daten($daten, 'veranstalter');
		$erg['veranstalter'] = $veranstalter[1];
		$daten = $this->_entferneDsv6Daten($daten, 'veranstalter');
		$ausrichter = $this->_holeDsv6Daten($daten, 'ausrichter');
		$erg['ausrichter'] = $ausrichter[1];
		$erg['absender'] = array(
			'name'    => $ausrichter[2],
			'strasse' => $ausrichter[3],
			'plz'     => $ausrichter[4],
			'ort'     => $ausrichter[5],
			'land'    => $ausrichter[6],
			'telefon' => $ausrichter[7],
			'fax'     => $ausrichter[8],
			'email'   => $ausrichter[9]
		);
		$daten = $this->_entferneDsv6Daten($daten, 'ausrichter');
		
		# Abschnitte
		$abschnitte = $this->_holeDsv6Daten($daten, 'abschnitt', false);
		$erg['abschnitte'] = array();
		foreach($abschnitte as $abschn){
			$erg['abschnitte'][$abschn[1]] = array(
				'datum' => $abschn[2],
				'zeit'  => $abschn[3]
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'abschnitt');
				
		$daten = $this->_entferneDsv6Daten($daten, 'kampfgericht');

		# Wettkaempfe
		$wettkaempfe = $this->_holeDsv6Daten($daten, 'wettkampf', false);
		$disziplinumsetzung = array('F' => 1,'R' => 3,'B' => 2,'S' => 4,'L' => 5,'X' => 6);
		$erg['wettkaempfe'] = array();
		$_staffel = array();
		$_einzel = array();
		foreach($wettkaempfe as $wk){
			if($wk[5] == 0 and $wk[6] == 0){
				continue;
			}
			$erg['wettkaempfe'][$wk[1]] = array(
				'nr'     => $wk[1],
				'art'    => $wk[2],
				'abschn' => $wk[3],
				'laenge' => ((!empty($wk[4]) and $wk[4]>1)?$wk[4].'x':'').$wk[5],
				'lage'   => $disziplinumsetzung[$wk[6]],
				'mw'     => strtolower($wk[8]),
				'blzuordn' => $wk[9]
			);
			if(!empty($wk[4]) and $wk[4]>1){
				$_staffel[] = $wk[1];
			}
			else{
				$_einzel[] = $wk[1];
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wettkampf');
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
		
		$wertungen = $this->_holeDsv6Daten($daten, 'wertung', false);
		$_wertungen = array();
		foreach($wertungen as $wertung){
			$_wertungen[$wertung[3]] = array('anzahl' => 0);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'wertung');
		
		# Vereinsbezeichnung
		$vereine = $this->_holeDsv6Daten($daten, 'verein', false);
		$_vereine = array();
		$erg['vereinsbezeichnung'] = array();
		$erg['vereinsbezeichnung'][0] = array(
			'name'           => 'Sonstige (nicht zuzuordnen)',
			'vnkennzahl'     => '9999',
			'landeschwverb'  => '99',
			'finanatkuerzel' => 'GER'
		);
		$i = 1;
		foreach($vereine as $verein){
			$verein[1] = trim($verein[1]);
			$_vereine[$verein[1]] = $i;
			$erg['vereinsbezeichnung'][$i] = array(
				'name'           => $verein[1],
				'vnkennzahl'     => $verein[2],
				'landeschwverb'  => $verein[3],
				'finanatkuerzel' => $verein[4]
			);
			$i++;
		}
		$daten = $this->_entferneDsv6Daten($daten, 'verein');
		
		$erg['ergebnisse'] = array();
		$einzelergebnisse = $this->_holeDsv6Daten($daten, 'pnergebnis', false);
		foreach($einzelergebnisse as $ergebnis){
			if($ergebnis[4] == 0){
				continue;
			}
			$_wertungen[$ergebnis[3]]['anzahl']++;
		}
		foreach($einzelergebnisse as $ergebnis){
			if(!isset($erg['ergebnisse'][$ergebnis[1]])){
				$erg['ergebnisse'][$ergebnis[1]] = array();
			}
			if(!isset($erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]])){
				$erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]] = array();
			}
			$erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]][] = array(
				'platz' => $ergebnis[4]==0?$ergebnis[5]:$ergebnis[4],
				'name'  => $ergebnis[6],
				'dsvid' => $ergebnis[7],
				'veranstid' => $ergebnis[8],
				'mw'    => strtolower($ergebnis[9]),
				'jahrg' => $ergebnis[10],
				'alterskl' => $ergebnis[11],
				'verein' => $this->_sucheVerein($ergebnis[12], $_vereine),
				'vereinid' => $ergebnis[13],
				'zwischenzeiten' => array(),
				'endzeit' => substr($ergebnis[14],3),
				'rekord' => '',
				'dsbem' => $ergebnis[15],
				'anzahl' => $_wertungen[$ergebnis[3]]['anzahl']
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'pnergebnis');
		
		$zwzeiten = $this->_holeDsv6Daten($daten, 'pnzwischenzeit', false);
		foreach($zwzeiten as $zwzeit){
			foreach($erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $zwzeit[1]){
					continue;
				}
				$erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]][$key]['zwischenzeiten'][$zwzeit[4]] = substr($zwzeit[5],3);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'pnzwischenzeit');
		$daten = $this->_entferneDsv6Daten($daten, 'pnreaktion');
	
		$staffelergebnisse = $this->_holeDsv6Daten($daten, 'stergebnis', false);
		foreach($staffelergebnisse as $ergebnis){
			if($ergebnis[4] == 0){
				continue;
			}
			$_wertungen[$ergebnis[3]]['anzahl']++;
		}
		foreach($staffelergebnisse as $ergebnis){
			if(!isset($erg['ergebnisse'][$ergebnis[1]])){
				$erg['ergebnisse'][$ergebnis[1]] = array();
			}
			if(!isset($erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]])){
				$erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]] = array();
			}
			$erg['ergebnisse'][$ergebnis[1]][$ergebnis[2]][] = array(
				'platz' => $ergebnis[4]==0?$ergebnis[5]:$ergebnis[4],
				'mannschaft'  => $ergebnis[6],
				'staffelklasse' => '',
				'veranstid' => $ergebnis[7],
				'verein' => $this->_sucheVerein($ergebnis[8], $_vereine),
				'vereinid' => $ergebnis[9],
				'zwischenzeiten' => array(),
				'endzeit' => substr($ergebnis[10],3),
				'schwimmer' => array(),
				'rekord' => '',
				'dsbem' => $ergebnis[12],
				'anzahl' => $_wertungen[$ergebnis[3]]['anzahl']
			);
		}
		$daten = $this->_entferneDsv6Daten($daten, 'stergebnis');
		
		$staffelpersonen = $this->_holeDsv6Daten($daten, 'staffelperson', false);
		foreach($staffelpersonen as $person){
			foreach($erg['ergebnisse'][$person[2]][$person[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $person[1]){
					continue;
				}
				$erg['ergebnisse'][$person[2]][$person[3]][$key]['schwimmer'][$person[6]] = array(
					'name'  => $person[4],
					'dsvid' => $person[5],
					'mw'    => strtolower($person[7]),
					'jahrg' => $person[8],
					'alterskl' => $person[9]
				);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'staffelperson');
		
		$zwzeiten = $this->_holeDsv6Daten($daten, 'stzwischenzeit', false);
		foreach($zwzeiten as $zwzeit){
			foreach($erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]] as $key => $ergebnis){
				if($ergebnis['veranstid'] != $zwzeit[1]){
					continue;
				}
				$erg['ergebnisse'][$zwzeit[2]][$zwzeit[3]][$key]['zwischenzeiten'][$zwzeit[5]] = substr($zwzeit[6],3);
			}
		}
		$daten = $this->_entferneDsv6Daten($daten, 'stzwischenzeit');
		$daten = $this->_entferneDsv6Daten($daten, 'stabloese');
	}
	else{
		# Ueberschrift, Dateiende
		$daten = str_replace('WETTKAMPFERGEBNISLISTE:','',$daten);
		$daten = str_replace('DATEIENDE','',$daten);
		# Format
		preg_match('~\s*FORMAT\s*:\s*(4|5)\s*;\s*(?:(ANSI|IBM)\s*;)?\s*~',$daten,$match);
		$erg['format'] = $match[1];
		$daten = str_replace($match[0],'',$daten);
		# Veranstaltung
		$regex = '~\s*VERANSTALTUNG\s*:\s*([^;]+)\s*;\s*([^;]+)\s*;(?:\s*(25|50|X)\s*;?\s*(HANDZEIT|AUTOMATISCH)\s*;)?\s*~';
		preg_match($regex, $daten, $match);
		if(isset($match[1])){
			$erg['veranstaltung'] = array(
				'bezeichnung' => $match[1],
				'ort'         => $match[2],
				'bahnlaenge'  => isset($match[3])?$match[3]:'',
				'zeitmessung' => isset($match[4])?$match[4]:''
			);
		}
		else{
			$erg['veranstaltung'] = array(
				'bezeichnung' => '[unbekannt]',
				'ort'         => '',
				'bahnlaenge'  => '',
				'zeitmessung' => ''
			);
		}
		$daten = preg_replace($regex, '', $daten);
		# Veranstalter, Ausrichter, Absender
		$regex = '~\s*VERANSTALTER\s*:\s*([^;]+)\s*;\s*AUSRICHTER\s*:\s*([^;]+)\s*;'; # Veranstalter, Absender
		$regex .= '\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*([^;]*)\s*;\s*~'; # Adresse
		preg_match($regex, $daten, $match);
		$erg['veranstalter'] = isset($match[1])?$match[1]:'';
		$erg['ausrichter'] = isset($match[2])?$match[2]:'';
		if(isset($match[3])){
			$erg['absender'] = array(
				'name'    => $match[3],
				'strasse' => $match[4],
				'plz'     => $match[5],
				'ort'     => $match[6],
				'land'    => $match[7],
				'telefon' => $match[8],
				'fax'     => $match[9],
				'email'   => $match[10]
			);
		}
		else{
			$erg['absender'] = array(
				'name'    => '',
				'strasse' => '',
				'plz'     => '',
				'ort'     => '',
				'land'    => '',
				'telefon' => '',
				'fax'     => '',
				'email'   => ''
			);
		}
	
		$daten = preg_replace($regex, '', $daten);
		# Abschnitte
		preg_match_all('~ABSCHNITT\s*:\s*(\d+)\s*;\s*(\d{2}\.\d{2}\.\d{4})\s*;\s*(\d{2}:\d{2})?\s*;~',$daten,$match);
		$erg['abschnitte'] = array();
		foreach($match[1] as $key => $abschn){
			$erg['abschnitte'][$abschn] = array(
				'datum' => $match[2][$key],
				'zeit'  => $match[3][$key]
			);
		}
		$daten = str_replace($match[0],'',$daten);
		$daten = trim($daten);
		# Wettkaempfe
		$regex = '~WETTKAMPF\s*:\s*(\d+)\s*;\s*(V|Z|F|A|E|N)?\s*;\s*(\d+)\s*;\s*((?:\d{1,2}x)?\d{1,5})\s*(F|R|B|S|L|X)\s*(W|M|X)\s*;';
		$regex .= '\s*(SW|MS|FS)\s*;';
		$regex .= '\s*~';
		$disziplinumsetzung = array(
			'F' => 1,
			'R' => 3,
			'B' => 2,
			'S' => 4,
			'L' => 5,
			'X' => 6
		);
		preg_match_all($regex,$daten,$match);
		$erg['wettkaempfe'] = array();
		$staffeln = array();
		$einzel = array();
		foreach($match[1] as $key => $wk){
			if($match[4][$key] == 0 and $match[5][$key] == 0){
				continue;
			}
			$erg['wettkaempfe'][$wk] = array(
				'nr'     => $match[1][$key],
				'art'    => $match[2][$key],
				'abschn' => $match[3][$key],
				'laenge' => $match[4][$key],
				'lage'   => $disziplinumsetzung[$match[5][$key]],
				'mw'     => strtolower($match[6][$key])
			);
			if($match[4][$key]{1} == 'x' or ((strlen($match[4][$key])>2) and $match[4][$key]{2} == 'x')){
				$staffeln[] = $wk;
			}
			else{
				$einzel[] = $wk;
			}
			$erg['wettkaempfe'][$wk]['blzuordn'] = $match[7][$key];
		}
		$daten = str_replace($match[0],'',$daten);
		$daten = trim($daten);
		
		if($ergebnisse == 'meta'){
			return $erg;
		}
	
		# Vereinsbezeichnung
		$erg['vereinsbezeichnung'] = array();
		$vereine = array();
		$erg['vereinsbezeichnung'][0] = array(
			'name'           => 'Sonstige (nicht zuzuordnen)',
			'vnkennzahl'     => '9999',
			'landeschwverb'  => '99',
			'finanatkuerzel' => 'GER'
		);
		$i = 1;
		$regex = '~VEREIN(?:SBEZEICHNUNG)?\s*:\s*([^;]+)\s*;\s*VNKENNZAHL\s*:\s*(\d{1,4})\s*;\s*LSV\s*:\s*([^;]+)?\s*;\s*NATION\s*:\s*([^;]+)\s*;\s*~';
		preg_match_all($regex,$daten,$match);
		foreach($match[1] as $key => $vn){
			$match[1][$key] = trim($match[1][$key]);
			$vereine[$match[1][$key]] = $i;
			$erg['vereinsbezeichnung'][$i] = array(
				'name'           => $this->_entferneNl($match[1][$key]),
				'vnkennzahl'     => $match[2][$key],
				'landeschwverb'  => ((strlen($match[3][$key])==2 and ctype_digit($match[3][$key]))?$match[3][$key]:''),
				'finanatkuerzel' => $match[4][$key]
			);
			$i++;
		}
		$daten = str_replace($match[0],'',$daten);
		$daten = trim($daten);
	
		# Ergebnisse
		$ergebnisse = explode('WK:',$daten);
		unset($ergebnisse[0]);
		$erg['ergebnisse'] = array();
		$regex_zeit = '(?:(?:\d{1,2}:\d{2},\d{2}\s*;\s*)|(?:\d{1,2]h\d{2}:\d{2}(?:,\d{2})*\s*;\s*))';
		$regex_pers = '~\s*([^;]+)\s*;\s*([^;]*)\s*;\s*(M|W)\s*;\s*(\d{1,4})\s*;\s*(\d*)\s*;\s*~';
		foreach($ergebnisse as $ergebnis){
			$regex = '~^\s*(\d+)\s*;\s*(V|Z|F|A|E|N)?\s*;\s*(.*)\s*$~s';
			preg_match($regex, $ergebnis, $match);
			if(!isset($erg['ergebnisse'][$match[1]])){
				$erg['ergebnisse'][$match[1]] = array();
			}
			if(empty($match[2])){
				$match[2] = '';
			}
			if(!isset($erg['ergebnisse'][$match[1]][$match[2]])){
				$erg['ergebnisse'][$match[1]][$match[2]] = array();
			}
			if(in_array($match[1], $staffeln)){ # Staffelergebnis
				if(empty($match[3])){
					continue;
				}
				$regex_s = '~(\d+|DS|NA|AB|AU|ZU)\s*;\s*(\d+)\s*;\s*((?:(?:JG:\s*\d+(?:-\d+)?)|(?:AK:\s*(?:\d+\+?|J|A|B|C|D|E)))?)\s*;\s*([^;]+)\s*;\s*(\d{1,4})\s*;\s*';
				$regex_s .= '('.$regex_zeit.'*)\s*('.$regex_zeit.')\s*(?:REKORD\s*:\s*(DR|DJR|DMR)\s*;\s*)?'; # Zeiten
				$regex_s .= '((?:\s*[^;]+\s*;\s*[0-9]*\s*;\s*(?:M|W)\s*;\s*\d{1,4}\s*;\s*\d*\s*;\s*)*)'; # Personen
				$regex_s .= '\s*(?:DSBEM\s*:\s*([^;]+)\s*;\s*)?'; # dsbem
				$regex_s .= '\s*~s';
				preg_match_all($regex_s, $match[3], $match_s);

				# Anzahl der Staffeln in der Wertung bestimmen
				$_anzahl = 0;
				foreach($match_s[1] as $platz){
					if(!in_array($platz,array('DS','NA','AB','AU'))){
						$_anzahl++;
					}
				}
				foreach($match_s[1] as $key => $wk){
					$schwimmer = array();
					preg_match_all($regex_pers, $match_s[9][$key], $match_pers);
					foreach($match_pers[1] as $key_pers => $pers){
						$schwimmer[] = array(
							'name'     => trim($match_pers[1][$key_pers]),
							'dsvid'    => $this->_sinnloserWert($match_pers[2][$key_pers], 'dsvid')?'':$match_pers[2][$key_pers],
							'mw'       => strtolower($match_pers[3][$key_pers]),
							'jahrg'    => $match_pers[4][$key_pers],
							'alterskl' => $match_pers[5][$key_pers],
						);
					}
					$zwzeiten = explode(';',substr($match_s[6][$key],0,-1));
					$anz = count($zwzeiten);
					for($i=0;$i<$anz;$i++){
						$zwzeiten[$i] = trim($zwzeiten[$i]);
					}
					$erg['ergebnisse'][$match[1]][$match[2]][] = array(
						'platz'          => $match_s[1][$key],
						'mannschaft'     => $match_s[2][$key],
						'staffelklasse'  => $match_s[3][$key],
						'verein'         => $this->_sucheVerein($match_s[4][$key], $vereine),
						'vereinid'       => $match_s[5][$key],
						'zwischenzeiten' => $zwzeiten,
						'endzeit'        => substr(trim($match_s[7][$key]),0,-1),
						'schwimmer'      => $schwimmer,
						'rekord'         => $match_s[8][$key],
						'dsbem'          => preg_replace("/(\015\012)|(\015)|(\012)/",' ',trim($match_s[10][$key])),
						'anzahl'         => $_anzahl
					);
				}
			}
			else{ # Einzelergebnis
				if(empty($match[3])){
					continue;
				}
				$regex_e = '~(\d+|DS|NA|AB|AU|ZU)\s*;\s*([^;]+)\s*;\s*([0-9]*)\s*;\s*(M|W)\s*;\s*(\d{1,4})\s*;\s*(\d*)\s*;\s*([^;]+)\s*;\s*(\d{1,4})\s*;\s*';
				$regex_e .= '('.$regex_zeit.'*)\s*('.$regex_zeit.')\s*(?:REKORD\s*:\s*(DR|DJR|DMR)\s*;\s*)?'; # Zeiten
				$regex_e .= '\s*(?:DSBEM\s*:\s*([^;]+)\s*;\s*)?'; # dsbem
				$regex_e .= '\s*~';
				preg_match_all($regex_e, $match[3], $match_e);
				# Anzahl der Teilnehmer in der Wertung bestimmen
				$_anzahl = 0;
				foreach($match_e[1] as $platz){
					if(!in_array($platz,array('DS','NA','AB','AU'))){
						$_anzahl++;
					}
				}
				foreach($match_e[1] as $key => $wk){
					$zwzeiten = explode(';',substr($match_e[9][$key],0,-1));
					$anz = count($zwzeiten);
					for($i=0;$i<$anz;$i++){
						$zwzeiten[$i] = trim($zwzeiten[$i]);
					}
					$erg['ergebnisse'][$match[1]][$match[2]][] = array(
						'platz'    => $match_e[1][$key],
						'name'     => trim($match_e[2][$key]),
						'dsvid'    => $this->_sinnloserWert($match_e[3][$key], 'dsvid')?'':$match_e[3][$key],
						'mw'       => strtolower($match_e[4][$key]),
						'jahrg'    => $match_e[5][$key],
						'alterskl' => $match_e[6][$key],
						'verein'   => $this->_sucheVerein($match_e[7][$key], $vereine),
						'vereinid' => $match_e[8][$key],
						'zwischenzeiten' => $zwzeiten,
						'endzeit'  => substr(trim($match_e[10][$key]),0,-1),
						'rekord'   => $match_e[11][$key],
						'dsbem'    => preg_replace("/(\015\012)|(\015)|(\012)/",' ',trim($match_e[12][$key])),
						'anzahl'   => $_anzahl
					);
				}
			}
		}
	}
	return $erg;
}

} # Ende der Klasse, EOF