<?php
function pruefen(&$wert,$art,$fehlertext,$sonstiges=''){
	$f = false;
	$fehlermeldung = '';
	$datumuhrzeitersetzen = array('.',',','-','_',' ',';',':','/','\\','+','*','#');
	switch($art){
	case 'jahr':
		if(strlen($wert) == 2){
			$wert = (empty($sonstiges)?'20':$sonstiges).$wert;
		}
		if(!preg_match('~^[0-9]{4}$~D', $wert) or $wert < 1800 or $wert > 2200){
			$f = true;
		}
		break;

	case 'geldbetrag':
		$tmp = preg_replace('~[^0-9]~',',',$wert);
		if(!empty($wert) and !preg_match('/^([0-9]+),?([0-9]{1,2})?$/D',$tmp)){
			$f = true;
		}
		else{
			preg_match('/^([0-9]+),?([0-9]{1,2})?$/D',$tmp,$match);
			$wert = sprintf('%d,%-02s',$match[1],$match[2]);
		}
		break;

	case 'schwimmzeit':
		if(empty($wert)){
			break;
		}
		$wert = trim($wert);
		$h = false;
		if(($hpos = strpos($wert, 'h')) !== false){
			$h = substr($wert, 0, $hpos);
			$wert = substr($wert, $hpos+1);
		}
		
		$zeit = preg_replace('~[^0-9]~',',',$wert);
		$teile = explode(',',$zeit);
		$ok = true;
		if(count($teile) == 1){
			$zeit = str_pad($teile[0], 6, '0', STR_PAD_LEFT);
			$min = substr($zeit,0,2);
			$sec = substr($zeit,2,2);
			$hdt = substr($zeit,4,2);
		}
		elseif((count($teile) == 3 and $zeit != ',,') or (count($teile) == 4 and $zeit != ',,,')){
			if(count($teile) == 3){
				list($min, $sec, $hdt) = $teile;
			}
			else{
				list($std, $min, $sec, $hdt) = $teile;
			}
			if(!empty($std)){
				$h = $std;	
			}
			if(empty($min)){
				$min = 0;
			}
			if(empty($sec)){
				$sec = 0;
			}
			if(empty($hdt)){
				$hdt = 0;
			}
		}
		else{
			$ok = false;
		}

		if($ok){
			if($min < 0){
				$ok = false;
			}
			if($min > 59){
				$h = floor($min/60);
				$min = $min%60;
			}
			if($sec < 0 or $sec > 59){
				$ok = false;
			}
			if($hdt < 0 or $hdt > 99){
				$ok = false;
			}
		}
		if($ok){
			$min = str_pad($min,2,'0',STR_PAD_LEFT);
			$sec = str_pad($sec,2,'0',STR_PAD_LEFT);
			$hdt = str_pad($hdt,2,'0',STR_PAD_LEFT);
			if($sonstiges == 'zeit'){
				$wert = ($h!==false?$h.'h':'').$min.':'.$sec.','.$hdt;
			}
			else{
				$wert = $min*60*100+$sec*100+$hdt;
				if($h !== false){
					$wert += $h*60*60*100;
				}
			}
		}
		else{
			$f = true;
		}
		break;

	case 'passwort':
		if(empty($wert)){
			break;
		}
		if(strlen($wert) < 8){
			$f = true;
			break;
		}
		if(!preg_match('~[0-9]~',$wert)){
			$f = true;
			break;
		}
		if(!preg_match('~[a-zäöüß]~',$wert)){
			$f = true;
			break;
		}
		if(!preg_match('~[A-ZÄÖÜ]~',$wert)){
			$f = true;
			break;
		}
		break;

	case 'telefon':
		if(empty($wert)){
			break;
		}
		$wert = preg_replace('~\+49[\D]*~', '0', $wert); # internationale Nummernangabe umwandeln
		$_t = preg_replace('~[\D]+~','/',$wert);
		$teile = explode('/',$_t);
		$anz = count($teile);
		if($anz == 1){
			$vorwahl = $sonstiges;
			$nummer = $wert;
		}
		elseif($anz == 0 OR $anz > 2){
			$f = true;
			break;
		}
		else{
			list($vorwahl,$nummer) = $teile;
		}
		if(!ctype_digit($vorwahl) and $vorwahl != '-h-'){
			$f = true;
			break;
		}
		if(!ctype_digit($nummer)){
			$f = true;
			break;
		}
		$wert = $vorwahl.'/'.$nummer;
		break;

	case 'wertemenge':
		if($wert==0){
			$f = true;
			foreach($sonstiges as $value){
				if($value === $wert){
					$f = false;
					break;
				}
			}
		}
		if(!empty($wert) and !in_array($wert,$sonstiges)){
			$f = true;
		}
		break;

	case 'leer':
		if(empty($wert) and $wert != '0'){
			$f = true;
		}
		break;
	case 'email':
		if(!empty($wert) and !preg_match('~^.+@.+\..{1,20}$~D',$wert)){
			$f = true;
		}
		break;
	case 'allesklein':
		if(strtolower($wert)!=$wert){
			$f = true;
		}
		break;
	case 'allesgroß':
		if(strtoupper($wert)!=$wert){
			$f = true;
		}
		break;
	case 'keinleerzeichen':
		if(str_replace(' ','',$wert)!=$wert){
			$f = true;
		}
		break;
	case 'zahl':
		if(!empty($wert) and !ctype_digit($wert)){
			$f = true;
		}
		break;
	case 'gleich':
		if($wert!=$sonstiges){
			$f = true;
		}
		break;
	case 'ungleich':
		if($wert==$sonstiges){
			$f = true;
		}
		break;
	case 'maxlaenge':
		if(strlen($wert)>$sonstiges){
			$f = true;
		}
		break;
	case 'minlaenge':
		if(!empty($wert) and strlen($wert)<$sonstiges){
			$f = true;
		}
		break;
	case 'regexpr':
	case 'regausdr':
		$wert = trim($wert);
		if(!empty($wert) and !preg_match('/'.$sonstiges.'/',$wert)){
			$f = true;
		}
		break;
	case 'datum':
		if(empty($wert)){
			break;
		}
		require_once(PFAD_INCLUDE.'func/datumerkennen.inc.php');
		if(empty($sonstiges)){
			$sonstiges = 'auto';
		}
		$f = datumerkennen($wert,$sonstiges)?false:true;
		break;

	case 'uhrzeit':
		if(empty($wert)){
			break;
		}
		if(strlen($wert)==4 or strlen($wert)==3 and ctype_digit($wert)){
			if(strlen($wert)==3){
				$wert = '0'.$wert;
			}
			$wert = substr($wert,0,2).':'.substr($wert,2,2);
		}
		$wert = preg_replace('~[\D]+~',':',$wert);
		$teile = explode(':',$wert);
		$anz = count($teile);
		if($anz != 1 and $anz != 2 AND $anz != 3){
			$f = true;
			break;
		}
		if(empty($teile[0])){
			$teile[0] = 0;
		}
		if(isset($teile[1]) and empty($teile[1])){
			$teile[1] = 0;
		}
		if(isset($teile[2]) and empty($teile[2])){
			$teile[2] = 0;
		}
		if($anz == 1){ # nur Stunden (HH)
			if(!ctype_digit($teile[0])){
				$f = true;
			}
			if($teile[0]<0 OR $teile[0]>23){
				$f = true;
			}
			$wert = substr('0'.$teile[0],-2).':00';
		}
		elseif($anz == 2){ # Stunden und Minuten (HH:MM)
			if(!ctype_digit($teile[0]) OR !ctype_digit($teile[1])){
				$f = true;
			}
			if($teile[1]<0 OR $teile[1]>59 OR $teile[0]<0 OR $teile[0]>23){
				$f = true;
			}
			$wert = substr('0'.$teile[0],-2).':'.substr('0'.$teile[1],-2);
		}
		else{ # Stunden, Minuten und Sekunden (HH:MM:SS)
			if(!ctype_digit($teile[0]) OR !ctype_digit($teile[1]) OR !ctype_digit($teile[2])){
				$f = true;
			}
			if($teile[1]<0 OR $teile[1]>59 OR $teile[2]<0 OR $teile[2]>59 OR $teile[0]<0 OR $teile[0]>23){
				$f = true;
			}
			$wert = substr('0'.$teile[0],-2).':'.substr('0'.$teile[1],-2).':'.substr('0'.$teile[2],-2);
		}
		break;
	case 'datumvordatum':
		// bei $wert nach $sonstiges -> Fehler
		if(!empty($sonstiges)){
			$von = preg_replace('~(\d{2})\.(\d{2})\.(\d{4})~',"$3$2$1",$wert);
			$nach = preg_replace('~(\d{2})\.(\d{2})\.(\d{4})~',"$3$2$1",$sonstiges);
			if($von > $nach){
				$f = true;
			}
		}
		break;
	}
	if($f) $fehlermeldung = $fehlertext.'<br />';
	return $fehlermeldung;
}
//EOF
?>